#!/bin/bash
#
# Copyright 2004-2016 RainbowFish Software
#
# PacsOne Server Installation Script

os=`uname -s`
echo "Checking for pre-requisites..."
if test $os = "Darwin"
then
macos=1
startbase="pacsone.macos"
libtool=`type -P glibtool 2>/dev/null`
libfreetype="libfreetype.dylib*"
ldd="otool -L"
else
kernel=`uname -r`
v=( ${kernel//-/ } )
v1=${v[0]}
v=( ${v1//./ } )
[ ${v[0]} -lt 2 ] && majorCheck=1 || majorCheck=0
[ ${v[0]} -eq 2 -a ${v[1]} -lt 6 ] && minorCheck=1 || minorCheck=0
[ ${v[0]} -eq 2 -a ${v[1]} -eq 6 -a ${v[2]} -lt 32 ] && revCheck=1 || revCheck=0
if [ $(($majorCheck + $minorCheck + $revCheck)) -gt 0 ]; then 
    echo "Kernel version 2.6.32 or later is required."
    exit
fi
macos=0
startbase="pacsone.base"
libtool=`type -P libtool 2>/dev/null`
libfreetype="libfreetype.so*"
ldd="ldd"
fi
if ! test -e "$libtool"
    then echo "libtool package is required but not installed."
    exit
fi
# some Linux distributions may not manage the symbolic link to the
# specific versions of libraries with major/monir numbers
found=0
for dir in "/usr/lib" "/usr/lib64"
do
check=`find $dir -name libfreetype.so* -print|wc -l`
if test $check -gt 0
then
    found=1
    break
fi
done
#if ! test -e /usr/lib/libfreetype.so
if test $found -eq 0
    then echo "libfreetype package is required but not installed."
    exit
fi
echo "OK."

# create the email key
echo `uname -a` > php/email.key

basedir=`pwd`;
if test $macos -ne 1
then    # Linux startup scripts
  chown root.daemon "$basedir/install.sh"
  chown root.daemon "$basedir/uninstall.sh"
  chown -R root.daemon "$basedir/php"
else    # why can't these *ix people agree on anything
  chown root:daemon "$basedir/install.sh"
  chown root:daemon "$basedir/uninstall.sh"
  chown -R root:daemon "$basedir/php"
fi
script="pacsone"
if test $macos -ne 1
then    # Linux startup scripts
  systemctl=`type -P systemctl 2>/dev/null`
  if test -e "$systemctl"
  then
    systemdBase="pacsone.service.base"
    systemdScript="pacsone.service"
    yes | cp -rf $basedir/$systemdBase $basedir/$systemdScript
    echo "ExecStart=$basedir/$script start" >> $basedir/$systemdScript
    echo "ExecStop=$basedir/$script stop" >> $basedir/$systemdScript
    echo "ExecReload=$basedir/$script restart" >> $basedir/$systemdScript
    chmod 755 $basedir/$systemdScript
  else
    chkconfig=`type -P chkconfig 2>/dev/null`
    if ! test -e "$chkconfig"
    then
      echo "Using default /sbin/chkconfig."
      chkconfig="/sbin/chkconfig"
      updatercd=`type -P update-rc.d 2>/dev/null`
      if ! test -e "$updatercd"
      then
        echo "Using default /usr/sbin/update-rc.d."
        updatercd="/usr/sbin/update-rc.d"
      fi # test -e updatercd
    fi # test -e $chkconfig
  fi # test -e $systemctl
fi

echo "#!/bin/sh" > $script
echo "#" >> $script
echo "# Copyright 2004-2016 RainbowFish Software" >> $script
echo "" >> $script
echo "# PacsOne Server start/stop script." >> $script
echo "" >> $script
echo "# Usually this is put in /etc/init.d (at least on machines SYSV R4 based" >> $script
echo "# systems) and linked to /etc/rc5.d/S99pacsone and /etc/rc0.d/K01pacsone" >> $script
echo "# If this is done the PacsOne Server will be started when the machine is" >> $script
echo "# started and shut down when the systems shuts down." >> $script
echo "#" >> $script
echo "# chkconfig: 35 99 00" >> $script
echo "# description: PacsOne Server" >> $script
echo "#" >> $script
echo "### BEGIN INIT INFO" >> $script
echo "# Provides:          pacsone" >> $script
echo "# Required-Start:    " >> $script
echo "# Required-Stop:     " >> $script
echo "# Default-Start:     3 5" >> $script
echo "# Default-Stop:      0 1 2 4 6" >> $script
echo "# Short-Description: PacsOne Server" >> $script
echo "# Description:       PacsOne Server" >> $script
echo "### END INIT INFO" >> $script
echo "" >> $script
echo "basedir=$basedir" >> $script
cat $basedir/$startbase >> $script
chmod 755 $script

# copy the control script and setup symbolic links in /etc/init.d/
echo "Configuring PacsOne Server Startup Scripts..."
if test $macos -ne 1
then    # Linux startup scripts
  # remove startup/shutdown scripts from previous versions
  if test -e /etc/rc.d/rc5.d/S99pacsone
  then rm -rf /etc/rc.d/rc5.d/S99pacsone
  fi
  if test -e /etc/rc.d/rc0.d/K01pacsone
  then rm -rf /etc/rc.d/rc0.d/K01pacsone
  fi
  # configure new startup/shutdown scripts
  if [ -d /etc/systemd/system/multi-user.target.wants/ ] && [ -x $systemctl ] && [ -d /lib/systemd/system/ ]
  then
    if ! test -e /lib/systemd/system/$systemdScript
    then cp $basedir/$systemdScript /lib/systemd/system/$systemdScript
    fi
    if ! test -e /etc/systemd/system/$systemdScript
    then ln -s /lib/systemd/system/$systemdScript /etc/systemd/system/$systemdScript
    fi
    if ! test -e /etc/systemd/system/multi-user.target.wants/$systemdScript
    then ln -s /lib/systemd/system/$systemdScript /etc/systemd/system/multi-user.target.wants/$systemdScript
    fi
    # refresh service list
    $systemctl daemon-reload
    # enable PacsOne Server service
    $systemctl -f enable pacsone
  else
    if test -d /etc/init.d/
    then
      initdir=/etc/init.d
    else
      initdir=/etc/rc.d
    fi
    if ! test -e $initdir/$script
    then
      ln -s $basedir/$script $initdir/$script
    fi
    if test -x $chkconfig
    then
      # run the 'chkconfig' command to setup the init scripts
      $chkconfig --add $script
    else
      # chkconfig is not available
      if test -x $updatercd
      then
        # run the 'update-rc.d' command to setup the init scripts
        $updatercd $script start 99 3 5 . stop 1 0 1 2 4 6 .
      else
        if ! test -e /etc/rc.d/rc5.d/S99pacsone
        then ln -s $initdir/$script /etc/rc.d/rc5.d/S99pacsone
        fi
        if ! test -e /etc/rc.d/rc0.d/K01pacsone
        then ln -s $initdir/$script /etc/rc.d/rc0.d/K01pacsone
        fi
      fi # test -e $updatercd
    fi # test -e $chkconfig
  fi # test -d /etc/systemd/system/multi-user.target.wants/
else    # Mac OS X startup scripts
  if test -d /Library/StartupItems/
  then
    initdir=/Library/StartupItems
  else
    initdir=/System/Library/StartupItems
  fi
  if ! test -d $initdir/$script
  then
    mkdir $initdir/$script
  fi
  # create symbolic links to starup items
  cp $basedir/$script $initdir/$script/$script
  chmod 755 $initdir/$script/$script
  cp $basedir/StartupParameters.plist $initdir/$script/StartupParameters.plist
  chmod 644 $initdir/$script/StartupParameters.plist
  chown root:wheel $initdir/$script/*
fi      # $macos -ne 1
echo "Done."

# copy the required libraries and setup symbolic links
echo "Configuring JPEG-LS Library..."
cd $basedir/CharLS
$libtool --mode=install /usr/bin/install -c libjpegls.la /usr/local/lib/libjpegls.la

echo "Configuring OpenSSL Libraries..."
check=`$ldd $basedir/PacsOne.exe|grep -i "not found"|grep -ic libssl.so`
if [ $check -gt 0 ] && [ ! -e /usr/local/lib/libssl.so.1.1 ];
then
  ln -s $basedir/OpenSSL/libssl.so.1.1 /usr/local/lib/libssl.so.1.1
  if ! test -e /usr/local/lib/libssl.so
  then
    ln -s /usr/local/lib/libssl.so.1.1 /usr/local/lib/libssl.so
  fi
fi
check=`$ldd $basedir/PacsOne.exe|grep -i "not found"|grep -ic libcrypto.so`
if [ $check -gt 0 ] && [ ! -e /usr/local/lib/libcrypto.so.1.1 ];
then
  ln -s $basedir/OpenSSL/libcrypto.so.1.1 /usr/local/lib/libcrypto.so.1.1
  if ! test -e /usr/local/lib/libcrypto.so
  then
    ln -s /usr/local/lib/libcrypto.so.1.1 /usr/local/lib/libcrypto.so
  fi
fi

echo "Configuring MySql Client Libraries..."
if ! test -e /usr/local/lib/libmariadb.so.3
then
  ln -s $basedir/mysql/libmariadb.so.3 /usr/local/lib/libmariadb.so.3
  ln -s /usr/local/lib/libmariadb.so.3 /usr/local/lib/libmariadb.so
fi
cd $basedir/mysql
$libtool --mode=install /usr/bin/install -c libmysqlpp.la /usr/local/lib/libmysqlpp.la
# LIBTOOL on some Linux distributions does not update the LD cache properly
if test $macos -ne 1
then
/sbin/ldconfig /usr/local/lib
# some Linux distributions lose the above ld path after reboot, so save it
# permanently in /etc/ld.so.conf
if test -d /etc/ld.so.conf.d/
then
    ldconf="/etc/ld.so.conf.d/pacsone-`uname -m`.conf"
else
    ldconf="/etc/ld.so.conf"
fi
# check if path is already set in configuration file
setldconf=1
if test -e $ldconf
then
    if test `grep "/usr/local/lib" $ldconf|wc -l` -gt 0
    then
        setldconf=0
    fi
fi
if test $setldconf -ne 0
then
    echo "Appending /usr/local/lib to $ldconf..."
    echo "/usr/local/lib" >> $ldconf
fi
fi  # $macos -ne 1
echo "Done."

echo "Configuring ImageMagick Libraries...";
cd $basedir/ImageMagick
cp -R ImageMagick-5.5.6-Q16 /usr/local/lib
find /usr/local/lib/ImageMagick-5.5.6-Q16 -name \*.so -exec chmod +x {} \;
$libtool --mode=install /usr/bin/install -c libMagick.la /usr/local/lib/libMagick.la
$libtool --finish -n /usr/local/lib
# LIBTOOL on some Linux distributions does not update the LD cache properly
if test $macos -ne 1
then
/sbin/ldconfig /usr/local/lib
fi

echo ""
cd $basedir/ImageMagick/ImageMagick-5.5.6-Q16/modules/coders
list='dcm.la gif.la jpeg.la ttf.la'; for p in $list; do \
if test -f $p; then \
    f="`echo $p | sed -e 's|^.*/||'`"; \
    echo "$f "
    $libtool --silent --mode=install /usr/bin/install -c $p /usr/local/lib/ImageMagick-5.5.6-Q16/modules/coders/$f; \
else :; fi; \
done
echo "Done."

# run PacsOne Server installer
echo ""
echo "Configuring PacsOne Server...";
cd $basedir
# check if all required libraries are present
check=`$ldd Setup.exe|grep -ic "not found"`
if test $check -gt 0
then
    echo "The following library is missing:"
    echo `$ldd Setup.exe|grep -i "not found"`
    exit
fi
$basedir/Setup.exe

echo "PacsOne Server Installation Complete.";
